@@ -1,17 +1,23 @@
-import argparse
 import os
 import pathlib
 
 from .. import cgns, h5m, vtu, xdmf
 from .._helpers import _filetype_from_path, read, reader_map
-from ._helpers import _get_version_text
 
 
-def decompress(argv=None):
-    # Parse command line arguments.
-    parser = _get_parser()
-    args = parser.parse_args(argv)
+def add_args(parser):
+    parser.add_argument("infile", type=str, help="mesh file to decompress")
+    parser.add_argument(
+        "--input-format",
+        "-i",
+        type=str,
+        choices=sorted(list(reader_map.keys())),
+        help="input file format",
+        default=None,
+    )
 
+
+def decompress(args):
     # read mesh data
     fmt = args.input_format or _filetype_from_path(pathlib.Path(args.infile))
 
@@ -37,30 +43,3 @@ def decompress(argv=None):
 
     size = os.stat(args.infile).st_size
     print(f"File size after:  {size / 1024 ** 2:.2f} MB")
-
-
-def _get_parser():
-    parser = argparse.ArgumentParser(
-        description=("Decompress mesh file."),
-        formatter_class=argparse.RawTextHelpFormatter,
-    )
-
-    parser.add_argument("infile", type=str, help="mesh file to decompress")
-
-    parser.add_argument(
-        "--input-format",
-        "-i",
-        type=str,
-        choices=sorted(list(reader_map.keys())),
-        help="input file format",
-        default=None,
-    )
-
-    parser.add_argument(
-        "--version",
-        "-v",
-        action="version",
-        version=_get_version_text(),
-        help="display version information",
-    )
-    return parser
