@@ -297,29 +297,46 @@ class Mesh:
     def sets_to_int_data(self):
         # If possible, convert cell sets to integer cell data. This is possible if all
         # cells appear exactly in one group.
-        intfun = []
-        for k, c in enumerate(zip(*self.cell_sets.values())):
-            # Go for -1 as the default value. (NaN is not int.)
-            arr = np.full(len(self.cells[k]), -1, dtype=int)
-            for i, cc in enumerate(c):
-                if cc is None:
-                    continue
-                arr[cc] = i
-            intfun.append(arr)
+        default_value = -1
+        if len(self.cell_sets) > 0:
+            intfun = []
+            for k, c in enumerate(zip(*self.cell_sets.values())):
+                # Go for -1 as the default value. (NaN is not int.)
+                arr = np.full(len(self.cells[k]), default_value, dtype=int)
+                for i, cc in enumerate(c):
+                    if cc is None:
+                        continue
+                    arr[cc] = i
+                intfun.append(arr)
 
-        data_name = "-".join(self.cell_sets.keys())
-        self.cell_data = {data_name: intfun}
-        self.cell_sets = {}
+            for item in intfun:
+                if np.any(item == default_value):
+                    warnings.warn(
+                        "Not all cells are part of a cell set. "
+                        f"Using default value {default_value}."
+                    )
+                    break
+
+            data_name = "-".join(self.cell_sets.keys())
+            self.cell_data = {data_name: intfun}
+            self.cell_sets = {}
 
         # now for the point sets
         # Go for -1 as the default value. (NaN is not int.)
-        intfun = np.full(len(self.points), -1, dtype=int)
-        for i, cc in enumerate(self.point_sets.values()):
-            intfun[cc] = i
+        if len(self.point_sets) > 0:
+            intfun = np.full(len(self.points), default_value, dtype=int)
+            for i, cc in enumerate(self.point_sets.values()):
+                intfun[cc] = i
 
-        data_name = "-".join(self.point_sets.keys())
-        self.point_data = {data_name: intfun}
-        self.point_sets = {}
+            if np.any(intfun == default_value):
+                warnings.warn(
+                    "Not all points are part of a point set. "
+                    f"Using default value {default_value}."
+                )
+
+            data_name = "-".join(self.point_sets.keys())
+            self.point_data = {data_name: intfun}
+            self.point_sets = {}
 
     def int_data_to_sets(self):
         """Convert all int data to {point,cell}_sets, where possible."""
@@ -335,8 +352,11 @@ class Mesh:
             tags = np.unique(np.concatenate(data))
 
             # try and get the names by splitting the key along "-" (this is how
-            # sets_to_int_data() forms the key
-            names = sorted(list(set(key.split("-"))))
+            # sets_to_int_data() forms the key)
+            names = key.split("-")
+            # remove duplicates and preserve order
+            # <https://stackoverflow.com/a/7961390/353337>:
+            names = list(dict.fromkeys(names))
             if len(names) != len(tags):
                 # alternative names
                 names = [f"set{tag}" for tag in tags]
@@ -364,7 +384,10 @@ class Mesh:
 
             # try and get the names by splitting the key along "-" (this is how
             # sets_to_int_data() forms the key
-            names = sorted(list(set(key.split("-"))))
+            names = key.split("-")
+            # remove duplicates and preserve order
+            # <https://stackoverflow.com/a/7961390/353337>:
+            names = list(dict.fromkeys(names))
             if len(names) != len(tags):
                 # alternative names
                 names = [f"set{tag}" for tag in tags]
